# Sessions

*NaifJs* is a multi-user dialog manager. 

The engine is not just a single-user dialog system.
Instead, it can be used as a "server" that manage dialogues with multiple users, in parallel.
Even if used as an embedded library, the engine can manage different conversational sessions at once.

## Session ids

An inbound request, the input sentence coming from a user, is identified by an *user id* (a pure number or any unique identifier string defined by the caller application).

> Note that *user id*, *session id*, or simply *id* are synonyms in NaifJs parlance.

```javascript
  let user = "134824744"
  let sentence = "I love you!" 

  naif.request(user, sentence)
```

## Session storage

To manage multi users at the same time, for each *session id* the engine store some dialog data 
(the *stateid* conversation state and more attributes) 
into a *session* data structure. 
Each session contains also dialog units variables. 

The sessions database is currently managed as in-memory object, 
disk-persistent (with a loaded/saved JSON file). 

When the engine bootstrap (`up`), checks if the session JSON file does exist and load it in memory.
When the engine stop working (`down`) the in-memory sessions are saved again on the JSON file.
By default, if not specified by developer, the engine look at / create a `sessions.json` in the application directory. 

> If *NaifJs* is used for a single-user application (e.g. an embedded system/IoT device), nothing change. 
> In this specific case the session storage will manage just a single entry.

Here below a session file example, 
containing entries for two users (session ids are in this case: `34824744` and `103034393`). 

```
$ cat examples/story_it/sessions.js
```
```json
{
  "34824744": {

    "unit": "secondUnit",
    "state": "restart",

    "time": 1602775249799,
    "active": false,

    "variables": {
      "firstUnit": {
        "firstName": "Giorgio"
      }
    },

    "response": {
      "text": "Arrivederci Giorgio!",
      "speech": false,
      "time": 1602775251566,
      "stateid": "secondUnit.restart"
    },

    "request": {
      "text": "no",
      "speech": false,
      "time": 1602775251565
    }

  },

  "103034393": {

    "unit": "ascoltaeripeti",
    "state": "restart",

    "time": 1561539134,
    "active": false,

    "response": {
      "text": "Arrivederci Giorgio!",
      "speech": false,
      "time": 1602775251577,
      "stateid": "ascoltaeripeti.restart"
    },

    "request": {
      "text": "no",
      "speech": false,
      "time": 1602775251000
    },

    "variables": {
      "leggieripeti": {
        "word": "TASCHE",
        "retryNum": 5,
        "score": 0
      },

      "ascoltaescrivi": {
        "word": "MERCATI",
        "retryNum": 5,
        "score": 0
      }
    }
  }
}  
```

---

[top](#) | [home](../README.md)
